from infi.shell.commands import commandgroup, command
from infi.shell.commands.types import Int
from .types import Name, Host, Cluster, Volume
from .host import HostCommandsBase
from . import render_collection

@commandgroup(prefix="cluster", provides=(Cluster,))
class ClusterCommands(HostCommandsBase):
    ENTITY_TYPE = Cluster
    @command(types=(Name,))
    def create(self, _api, name):
        return self._create(_api, name=name)
    @command(types=(Cluster, Name),
             substitute=dict(cluster="name"))
    def rename(self, _api, cluster, new_name):
        return self._rename(_api, cluster, new_name)
    @command(types=(Cluster,),
             substitute=dict(cluster="name"))
    def delete(self, _api, cluster):
        return self._delete(_api, cluster)
    @command(types=(Cluster, Host),
             substitute=dict(cluster="name"))
    def add_host(self, _api, cluster, host):
        _api.post("clusters/{}/hosts".format(cluster.id), dict(id=host.id))
        return 'Host "{}" added to cluster "{}"'.format(host.name, cluster.name)
    @command(types=(Cluster, Host),
             substitute=dict(cluster="name"))
    def remove_host(self, _api, cluster, host):
        _api.delete("clusters/{}/hosts/{}".format(cluster.id, host.id))
        return 'Host "{}" removed from cluster "{}"'.format(host.name, cluster.name)
    @command(types=(Cluster, Volume, Int),
             substitute=dict(cluster="name"))
    def map(self, _api, cluster, volume, lun=None):
        return self._map(_api, cluster, volume, lun)
    @command(types=(Cluster, Volume, Int),
             substitute=dict(cluster="name"))
    def unmap(self, _api, cluster, volume=None, lun=None):
        return self._unmap(_api, cluster, volume, lun)
    @command(types=(Cluster,),
             substitute=dict(cluster="name"))
    @render_collection(entity=Cluster, order=["name", "hosts", "luns"])
    def query(self, _api, cluster=None):
        return self._query(_api, cluster, _hook=self._query_hook)
    def _query_hook(self, clusters):
        for cluster in clusters:
            cluster.hosts = [host.name for host in cluster.hosts]
            cluster.luns = [lun.lun for lun in cluster.luns]
        return clusters
    @command(types=(Cluster, Volume),
             substitute=dict(cluster="name"))
    @render_collection(entity=Cluster, alignment="left", order=["cluster"])
    def map_query(self, _api, cluster=None, volume=None):
        return self._map_query(_api, cluster, volume)
