from . import EntityCommands
from infi.shell.commands import commandgroup, command
from .types import Name, Pool, Volume, ConsGroup, SnapSet, ISRPolicy
from . import Null

@commandgroup(prefix="consgroup", provides=(ConsGroup,))
class ConsGroupCommands(EntityCommands):
    ENTITY_TYPE = ConsGroup
    @command(types=(Name,))
    def create(self, _api, name):
        return self._create(_api, name=name)
    @command(types=(ConsGroup, Volume))
    def add_volume(self, _api, consgroup, volume):
        volume_id = _api.post("consgroups/{}/volumes".format(consgroup.id), volume.id)
        return "Added"
    @command(types=(ConsGroup, Volume))
    def remove_volume(self, _api, consgroup, volume):
        volume_id = _api.delete("consgroups/{}/volumes/{}".format(consgroup.id, volume.id))
        return "Removed"
    @command(types=(ConsGroup,))
    def disband(self, _api, consgroup):
        return self._update(_api, consgroup, "Disbanded", volumes=[])
    @command(types=(ConsGroup, Pool))
    def change_pool(self, _api, consgroup, new_pool):
        return self._update(_api, consgroup, "Moved", pool_id=new_pool.id)
    @command(types=(ConsGroup,))
    def delete(self, _api, consgroup):
        return self._delete(_api, consgroup)
    @command(types=(ConsGroup, Name, Name))
    def snap(self, _api, consgroup, name, suffix=None):
        props = dict(parent_id=consgroup.id, name=name, suffix=suffix)
        return _api.post("snapsets", self._filter_properties(props))
    @command(types=(ConsGroup, SnapSet))
    def restore(self, _api, consgroup, snapset):
        return self._update(_api, consgroup, "Restored", data=snapset.data)
    @command(types=(ConsGroup, Name))
    def rename(self, _api, consgroup, new_name):
        return self._rename(_api, consgroup, new_name)
    @command(types=(ConsGroup,))
    def query(self, _api, consgroup=None):
        return self._query(_api, consgroup)
    @command(types=(ConsGroup, ISRPolicy))
    def set_snapshot_rotation(self, _api, consgroup, policy):
        return self._update(_api, consgroup, returns="Policy set", snapshot_rotation=policy)
    @command(types=(ConsGroup,))
    def clear_snapshot_rotation(self, _api, consgroup):
        return self._update(_api, consgroup, returns="Policy cleared", snapshot_rotation=Null)

