from infi.shell.commands import commandgroup, command, render
from infi.shell.commands.types import Int

from .types import Drive
from . import superuser, admin_only

QUERY_ORDER = ["index", "state", "vendor", "model", "product_rev", "serial_number", "firmware"]
QUERY_EXCLUDED = "DRIVE_INDEX ENCLOSURE_INDEX HW STATE_DESCRIPTION NODES_ACCESS PRODUCT_ID LAST_PROBE_TIMESTAMP PROBE_TTL".split()
QUERY_ADMIN_EXCLUDED = "vendor model product_rev firmware serial_number".split()
QUERY_SUBSTITUTES = dict(active_time="active_since", bytes_capacity="capacity", product_rev="revision")

@commandgroup(prefix="drive")
class DriveCommands(object):
    @superuser
    @command
    @render(order=QUERY_ORDER,
            excluded=QUERY_EXCLUDED,
            substitute=QUERY_SUBSTITUTES,
            )
    def query(self, _api):
        return Drive.get_all(_api)
    @admin_only
    @command(name="query")
    @render(order=QUERY_ORDER,
            excluded=QUERY_EXCLUDED + QUERY_ADMIN_EXCLUDED,
            substitute=QUERY_SUBSTITUTES,
            )
    def admin__query(self, _api):
        return Drive.get_all(_api)
    @command(types=(Int, Int))
    def phase_in(self, _api, enc_index, drive_index):
        self._drive_action(_api, enc_index, drive_index, "phase_in")
        return "Phased-in enclosure {} drive {}".format(enc_index, drive_index)
    @command(types=(Int, Int))
    def phase_out(self, _api, enc_index, drive_index):
        self._drive_action(_api, enc_index, drive_index, "phase_out")
        return "Phased-out enclosure {} drive {}".format(enc_index, drive_index)
    @command(types=(Int, Int))
    def test(self, _api, enc_index, drive_index):
        self._drive_action(_api, enc_index, drive_index, "test")
        return "Tested drive {} of enclosure {}".format(drive_index, enc_index)
    def _drive_action(self, _api, enc_index, drive_index, state):
        _api.post("components/enclosures/{}/drives/{}/{}".format(enc_index, drive_index, state), None)
