from infi.shell.commands import commandgroup, command, render

from .types import Enclosure
from . import superuser, admin_only

QUERY_ORDER = ["state", "vendor", "model", "revision", "firmware"]
QUERY_EXCLUDED = ["STATE_DESCRIPTION"]
QUERY_ADMIN_EXCLUDED = "vendor model revision firmware probe_ttl".split()
QUERY_SUBSTITUTES = dict(drives_number="drives")

@commandgroup(prefix="enclosure")
class EnclosureCommands(object):
    @superuser
    @command
    @render(order=QUERY_ORDER,
            excluded=QUERY_EXCLUDED,
            substitute=QUERY_SUBSTITUTES)
    def query(self, _api):
        return Enclosure.iter_all(_api, hook=self._query_hook)
    @admin_only
    @command(name="query")
    @render(order=QUERY_ORDER,
            excluded=QUERY_EXCLUDED + QUERY_ADMIN_EXCLUDED,
            substitute=QUERY_SUBSTITUTES)
    def admin__query(self, _api):
        return Enclosure.iter_all(_api, hook=self._query_hook)
    def _query_hook(self, enclosures):
        for e in enclosures:
            del e["drives"]
        return enclosures
