from bunch import Bunch

from infi.shell.commands import commandgroup, command, render

from .types import Node
from . import superuser, admin_only

QUERY_ORDER = ["name", "state", "vendor", "model", "core_state", "core_role", "mgmt_state", "mgmt_role"]
QUERY_EXCLUDED = []
QUERY_ADMIN_EXCLUDED = ["vendor", "model"]

@commandgroup(prefix="node")
class NodeCommands(object):
    def _query(self, _api):
        return [self._get_node_info(node) for node in Node.get_all(_api)]
    @superuser
    @command
    @render(
        order=QUERY_ORDER,
        excluded=QUERY_EXCLUDED,
        )
    def query(self, _api):
        return self._query(_api)
    @admin_only
    @command(name="query")
    @render(
        order=QUERY_ORDER,
        excluded=QUERY_EXCLUDED + QUERY_ADMIN_EXCLUDED,
        )
    def admin__query(self, _api):
        return self._query(_api)
    def _get_node_info(self, node):
        services = Bunch((s.name, s) for s in node.services)
        return Bunch(name=node.name,
                     state=node.state,
                     vendor=node.vendor,
                     model=node.model,
                     core_state=services.core.state,
                     core_role=services.core.role,
                     mgmt_state=services.mgmt.state,
                     mgmt_role=services.mgmt.role,
                     )
    @command(types=(Node,))
    def phase_in(self, _api, node):
        _api.post("components/nodes/{}/phase_in".format(node.id))
        return "Phasing in {}".format(node.name)
    @command(types=(Node,))
    def phase_out(self, _api, node):
        _api.post("components/nodes/{}/phase_out".format(node.id))
        return "Phasing out {}".format(node.name)
