from . import EntityCommands, render_collection, superuser
from infi.shell.commands import commandgroup, command, render
from .types import Name, Size, Pool
import functools

@commandgroup(prefix="pool")
class PoolCommands(EntityCommands):
    ENTITY_TYPE = Pool
    
    @superuser
    @command(types=(Name, Size, Size))
    def create(self, _api, name, soft_capacity, hard_capacity):
        return self._create(_api, name=name, soft_capacity=soft_capacity, hard_capacity=hard_capacity)

    @superuser
    @command(types=(Pool,),
             substitute=dict(pool="name"))
    @render_collection(entity=Pool, order=["name", "volume_count"])
    def query(self, _api, pool=None):
        return self._query(_api, pool)

    @superuser
    @command(types=(Pool.PARTIAL, Name),
             substitute=dict(pool="name"))
    def rename(self, _api, pool, new_name):
        return self._rename(_api, pool, new_name)

    @superuser
    @command(types=(Pool, Size, Size),
             substitute=dict(pool="name"))
    def update(self, _api, pool, soft_capacity=None, hard_capacity=None):
        update_soft = functools.partial(self._update, _api, pool, soft_capacity=soft_capacity)
        update_hard = functools.partial(self._update, _api, pool, hard_capacity=hard_capacity)
        if hard_capacity < pool.hard_capacity:
            update_hard()
            update_soft()
        else:
            update_soft()
            update_hard()
        return self._result_message(pool, "updated")

    @superuser
    @command(types=(Pool.PARTIAL,),
             substitute=dict(pool="name"))
    def delete(self, _api, pool):
        return self._delete(_api, pool)

