from infi.shell.exceptions import CommandFailed

class InvalidOptions(Exception):
    def __init__(self, msg, print_help=False, print_usage=False):
        super(InvalidOptions, self).__init__(msg)
        self.print_help, self.print_usage = print_help, print_usage

class APIFailure(CommandFailed):
    pass

class UnexpectedAPIResponse(APIFailure):
    pass

class ConnectionFailed(APIFailure):
    pass

class ManagementException(APIFailure):
    def __init__(self, code, message, http_code):
        self.code = code
        self.message = message
        self.http_code = http_code
        super(ManagementException, self).__init__(self.message)

class Unauthorized(APIFailure):
    def __init__(self, *_):
        super(Unauthorized, self).__init__("Bad credentials")

class BadURI(APIFailure):
    pass

class ConfigurationConflict(APIFailure):
    pass

class InternalServerError(APIFailure):
    pass

class NotSupported(APIFailure):
    pass

class ServiceTemporarilyUnavailable(APIFailure):
    pass

class Unauthorized(APIFailure):
    pass

class Forbidden(APIFailure):
    pass

class UnexpectedServerResponse(APIFailure):
    def __init__(self, http_code, msg=None):
        if msg:
            msg = "Server returned an unexpected HTTP code ({}) with the following error: {}".format(http_code, msg)
        else:
            msg = "Server returned an unexpected HTTP code: {}".format(http_code)
        super(UnexpectedServerResponse, self).__init__(msg)

class BadPageStructure(CommandFailed):
    pass

class NoSuchEntity(APIFailure):
    pass

class InvalidConstant(CommandFailed):
    pass
