from itertools import chain

class PossiblyNone(object):
    class __metaclass__(type):
        def __new__(mcls, name, bases, attrs):
            base_methods = 'add sub mul div truediv floordiv mod divmod pow lshift rshift and xor or'.split()
            rbase_methods = ['r'+m for m in base_methods]
            ibase_methods = ['i'+m for m in base_methods]
            other_methods = 'neg pos abs invert complex int long float oct hex'.split()
            methods = chain(base_methods, rbase_methods, ibase_methods, other_methods)

            underscores = lambda m: '__{}__'.format(m)

            def create_operator(method_name): # This is a method because method_name must not change
                def operator(self, *args):
                    if self._is_none:
                        return self
                    args = list(args)
                    invoked_operand = self._value
                    actual_method_name = method_name
                    if method_name in base_methods and args and isinstance(args[0], type(self)):
                        actual_method_name = 'r' + actual_method_name
                        invoked_operand, args[0] = args[0], invoked_operand
                    actual_method_name = underscores(actual_method_name)
                    real_op_method = getattr(invoked_operand, actual_method_name, None)
                    if real_op_method is None:
                        raise NotImplementedError
                    return real_op_method(*args)
                operator.__name__ = method_name
                return operator

            for op_method_name in methods:
                attrs[underscores(op_method_name)] = create_operator(op_method_name)
            return type.__new__(mcls, name, bases, attrs)

    def __init__(self, value, alternative_value='N/A'):
        self._is_none = value is None
        self._value = value
        self._alternative_value = unicode(alternative_value)
        def __setattr__(self, attr, value):
            if not self._is_none:
                setattr(self._value, attr, value)
        self.__setattr__ = __setattr__
    def __repr__(self):
        return '<PossiblyNone: {}>'.format(repr(self._value))
    def __str__(self):
        return self._alternative_value if self._is_none else str(self._value)
    def __unicode__(self):
        return self._alternative_value if self._is_none else unicode(self._value)
    def __nonzero__(self):
        return True if self._is_none else bool(self._value)
    def __getattr__(self, attr):
        return self if self._is_none else getattr(self._value, attr)
    def __hasattr__(self, attr):
        return False if self._is_none else hasattr(self._value, attr)
    def __getitem__(self, item):
        return self if self._is_none else self._value[item]
    def __setitem__(self, item, value):
        if not self._is_none:
            self._value[item] = value
    def __delitem__(self, item):
        if not self._is_none:
            del self._value[item]
    def __len__(self):
        return 0 if self._is_none else len(self._value)
    def __iter__(self):
        return iter(()) if self._is_none else iter(self._value)
    def __reversed__(self):
        return self if self._is_none else reversed(self._value)
    def __contains__(self, value):
        return False if self._is_none else value in self._value

    @classmethod
    def wrap_items(cls, container, **kwargs):
        if isinstance(container, dict):
            return type(container)((key, cls.wrap_items(value, **kwargs)) for key, value in container.iteritems())
        elif isinstance(container, (list, tuple, set)):
            return type(container)(cls.wrap_items(value, **kwargs) for value in container)
        else:
            # Container is actually the value here
            return cls(container, **kwargs)
